@@ -1,6 +1,8 @@
 from . import base
 from . import antique
+from . import aol_ia
 from . import aquaint
+from . import argsme
 from . import beir
 from . import c4
 from . import car
@@ -27,6 +29,7 @@ from . import nfcorpus
 from . import natural_questions
 from . import nyt
 from . import pmc
+from . import touche
 from . import trec_arabic
 from . import trec_mandarin
 from . import trec_spanish
