@@ -1,10 +1,10 @@
 import json
 import codecs
-from typing import NamedTuple, Dict
+from typing import NamedTuple, Dict, List
 import ir_datasets
-from ir_datasets.util import ZipExtract, Cache, Lazy
+from ir_datasets.util import ZipExtract, Cache, Lazy, Migrator
 from ir_datasets.datasets.base import Dataset, YamlDocumentation, FilteredQueries
-from ir_datasets.formats import BaseQueries, BaseDocs, BaseQrels, TrecQrel
+from ir_datasets.formats import BaseQueries, BaseDocs, BaseQrels, GenericDoc, GenericQuery, TrecQrel
 from ir_datasets.indices import PickleLz4FullStore
 
 _logger = ir_datasets.log.easy()
@@ -20,18 +20,92 @@ class BeirDoc(NamedTuple):
     metadata: Dict[str, str]
 
 
-class BeirQuery(NamedTuple):
+class BeirTitleDoc(NamedTuple):
+    doc_id: str
+    text: str
+    title: str
+
+class BeirTitleUrlDoc(NamedTuple):
+    doc_id: str
+    text: str
+    title: str
+    url: str
+
+class BeirSciDoc(NamedTuple):
+    doc_id: str
+    text: str
+    title: str
+    authors: List[str]
+    year: int
+    cited_by: List[str]
+    references: List[str]
+
+class BeirCordDoc(NamedTuple):
+    doc_id: str
+    text: str
+    title: str
+    url: str
+    pubmed_id: str
+
+class BeirToucheDoc(NamedTuple):
+    doc_id: str
+    text: str
+    title: str
+    stance: str
+    url: str
+
+class BeirCqaDoc(NamedTuple):
+    doc_id: str
+    text: str
+    title: str
+    tags: List[str]
+
+class BeirUrlQuery(NamedTuple):
     query_id: str
     text: str
-    metadata: Dict[str, str]
+    url: str
 
+class BeirSciQuery(NamedTuple):
+    query_id: str
+    text: str
+    authors: List[str]
+    year: int
+    cited_by: List[str]
+    references: List[str]
+
+class BeirToucheQuery(NamedTuple):
+    query_id: str
+    text: str
+    description: str
+    narrative: str
+
+class BeirCovidQuery(NamedTuple):
+    query_id: str
+    text: str
+    query: str
+    narrative: str
+
+class BeirCqaQuery(NamedTuple):
+    query_id: str
+    text: str
+    tags: List[str]
+
+def _map_field(field, data):
+    if field in ('doc_id', 'query_id'):
+        return data['_id']
+    if field == 'text':
+        return data['text']
+    if field == 'title':
+        return data['title']
+    else:
+        return data['metadata'][field]
 
 class BeirDocs(BaseDocs):
-    def __init__(self, name, dlc, count_hint=None):
+    def __init__(self, name, dlc, doc_type):
         super().__init__()
         self._name = name
         self._dlc = dlc
-        self._count_hint = count_hint
+        self._doc_type = doc_type
 
     def docs_iter(self):
         return iter(self.docs_store())
@@ -40,10 +114,10 @@ class BeirDocs(BaseDocs):
         with self._dlc.stream() as stream:
             for line in stream:
                 data = json.loads(line)
-                yield BeirDoc(data['_id'], data['text'], data.get('title', ''), data.get('metadata', {}))
+                yield self._doc_type(*(_map_field(f, data) for f in self._doc_type._fields))
 
     def docs_cls(self):
-        return BeirDoc
+        return self._doc_type
 
     def docs_store(self, field='doc_id'):
         return PickleLz4FullStore(
@@ -52,11 +126,12 @@ class BeirDocs(BaseDocs):
             data_cls=self.docs_cls(),
             lookup_field=field,
             index_fields=['doc_id'],
-            count_hint=self._count_hint,
+            count_hint=ir_datasets.util.count_hint(f'{NAME}/{self._name}'),
         )
 
     def docs_count(self):
-        return self.docs_store().count()
+        if self.docs_store().built():
+            return self.docs_store().count()
 
     def docs_namespace(self):
         return f'{NAME}/{self._name}'
@@ -66,23 +141,20 @@ class BeirDocs(BaseDocs):
 
 
 class BeirQueries(BaseQueries):
-    def __init__(self, name, dlc, keep_metadata=None):
+    def __init__(self, name, dlc, query_type):
         super().__init__()
         self._name = name
         self._dlc = dlc
-        self._keep_metadata = keep_metadata
+        self._query_type = query_type
 
     def queries_iter(self):
         with self._dlc.stream() as stream:
             for line in stream:
                 data = json.loads(line)
-                metadata = data.get('metadata', {})
-                if self._keep_metadata is not None:
-                    metadata = {k: v for k, v in metadata.items() if k in self._keep_metadata}
-                yield BeirQuery(data['_id'], data['text'], metadata)
+                yield self._query_type(*(_map_field(f, data) for f in self._query_type._fields))
 
     def queries_cls(self):
-        return BeirQuery
+        return self._query_type
 
     def queries_namespace(self):
         return f'{NAME}/{self._name}'
@@ -130,31 +202,36 @@ def _init():
     subsets = {}
 
     benchmarks = {
-        'msmarco': (['train', 'dev', 'test'], 8841823),
-        'trec-covid': (['test'], 171332),
-        'nfcorpus': (['train', 'dev', 'test'], 3633),
-        'nq': (['test'], 2681468),
-        'hotpotqa': (['train', 'dev', 'test'], 5233329),
-        'fiqa': (['train', 'dev', 'test'], 57638),
-        'arguana': (['test'], 8674),
-        'webis-touche2020': (['test'], 382545),
-        'quora': (['dev', 'test'], 522931),
-        'dbpedia-entity': (['dev', 'test'], 4635922),
-        'scidocs': (['test'], 25657),
-        'fever': (['train', 'dev', 'test'], 5416568),
-        'climate-fever': (['test'], 5416593),
-        'scifact': (['train', 'test'], 5183),
+        'msmarco': (['train', 'dev', 'test'], GenericDoc, GenericQuery),
+        'trec-covid': (['test'], BeirCordDoc, BeirCovidQuery),
+        'nfcorpus': (['train', 'dev', 'test'], BeirTitleUrlDoc, BeirUrlQuery),
+        'nq': (['test'], BeirTitleDoc, GenericQuery),
+        'hotpotqa': (['train', 'dev', 'test'], BeirTitleUrlDoc, GenericQuery),
+        'fiqa': (['train', 'dev', 'test'], GenericDoc, GenericQuery),
+        'arguana': (['test'], BeirTitleDoc, GenericQuery),
+        'webis-touche2020': (['test'], BeirToucheDoc, BeirToucheQuery),
+        'webis-touche2020/v2': (['test'], BeirToucheDoc, BeirToucheQuery),
+        'quora': (['dev', 'test'], GenericDoc, GenericQuery),
+        'dbpedia-entity': (['dev', 'test'], BeirTitleUrlDoc, GenericQuery),
+        'scidocs': (['test'], BeirSciDoc, BeirSciQuery),
+        'fever': (['train', 'dev', 'test'], BeirTitleDoc, GenericQuery),
+        'climate-fever': (['test'], BeirTitleDoc, GenericQuery),
+        'scifact': (['train', 'test'], BeirTitleDoc, GenericQuery),
     }
 
-    for ds, (qrels, count_hint) in benchmarks.items():
+    for ds, (qrels, doc_type, query_type) in benchmarks.items():
         dlc_ds = dlc[ds]
-        docs = BeirDocs(ds, ZipExtract(dlc_ds, f'{ds}/corpus.jsonl'), count_hint=count_hint)
-        queries = BeirQueries(ds, Cache(ZipExtract(dlc_ds, f'{ds}/queries.jsonl'), base_path/ds/'queries.json'))
+        ds_zip = ds.split('/')[0]
+        docs_migrator = Migrator(base_path/ds/'irds_version.txt', 'v2',
+            affected_files=[f'{base_path/ds}/docs.pklz4'],
+            message=f'Migrating {NAME}/{ds} (structuring fields)')
+        docs = docs_migrator(BeirDocs(ds, ZipExtract(dlc_ds, f'{ds_zip}/corpus.jsonl'), doc_type))
+        queries = BeirQueries(ds, Cache(ZipExtract(dlc_ds, f'{ds_zip}/queries.jsonl'), base_path/ds/'queries.json'), query_type)
         if len(qrels) == 1:
             subsets[ds] = Dataset(
                 docs,
                 queries,
-                BeirQrels(Cache(ZipExtract(dlc_ds, f'{ds}/qrels/{qrels[0]}.tsv'), base_path/ds/f'{qrels[0]}.qrels'), qrels_defs={}),
+                BeirQrels(Cache(ZipExtract(dlc_ds, f'{ds_zip}/qrels/{qrels[0]}.tsv'), base_path/ds/f'{qrels[0]}.qrels'), qrels_defs={}),
                 documentation(ds)
             )
         else:
@@ -164,7 +241,7 @@ def _init():
                 documentation(ds)
             )
             for qrel in qrels:
-                subset_qrels = BeirQrels(Cache(ZipExtract(dlc_ds, f'{ds}/qrels/{qrel}.tsv'), base_path/ds/f'{qrel}.qrels'), qrels_defs={})
+                subset_qrels = BeirQrels(Cache(ZipExtract(dlc_ds, f'{ds_zip}/qrels/{qrel}.tsv'), base_path/ds/f'{qrel}.qrels'), qrels_defs={})
                 subset_qids = qid_filter(subset_qrels)
                 subsets[f'{ds}/{qrel}'] = Dataset(
                     docs,
@@ -173,25 +250,15 @@ def _init():
                     documentation(f'{ds}/{qrel}')
                 )
 
-    cqa = [
-        ('android', 22998),
-        ('english', 40221),
-        ('gaming', 45301),
-        ('gis', 37637),
-        ('mathematica', 16705),
-        ('physics', 38316),
-        ('programmers', 32176),
-        ('stats', 42269),
-        ('tex', 68184),
-        ('unix', 47382),
-        ('webmasters', 17405),
-        ('wordpress', 48605),
-    ]
+    cqa = ['android', 'english', 'gaming', 'gis', 'mathematica', 'physics', 'programmers', 'stats', 'tex', 'unix', 'webmasters', 'wordpress']
     cqa_dlc = dlc['cqadupstack']
-    for ds, count_hint in cqa:
+    for ds in cqa:
+        docs_migrator = Migrator(base_path/'cqadupstack'/ds/'irds_version.txt', 'v2',
+            affected_files=[f'{base_path/"cqadupstack"/ds}/docs.pklz4'],
+            message=f'Migrating {NAME}/cqadupstack/{ds} (structuring fields)')
         subsets[f'cqadupstack/{ds}'] = Dataset(
-            BeirDocs(f'cqadupstack/{ds}', ZipExtract(cqa_dlc, f'cqadupstack/{ds}/corpus.jsonl'), count_hint=count_hint),
-            BeirQueries(f'cqadupstack/{ds}', Cache(ZipExtract(cqa_dlc, f'cqadupstack/{ds}/queries.jsonl'), base_path/'cqadupstack'/ds/'queries.json'), keep_metadata=['tags']),
+            docs_migrator(BeirDocs(f'cqadupstack/{ds}', ZipExtract(cqa_dlc, f'cqadupstack/{ds}/corpus.jsonl'), BeirCqaDoc)),
+            BeirQueries(f'cqadupstack/{ds}', Cache(ZipExtract(cqa_dlc, f'cqadupstack/{ds}/queries.jsonl'), base_path/'cqadupstack'/ds/'queries.json'), BeirCqaQuery),
             BeirQrels(Cache(ZipExtract(cqa_dlc, f'cqadupstack/{ds}/qrels/test.tsv'), base_path/'cqadupstack'/ds/f'test.qrels'), qrels_defs={}),
             documentation(f'cqadupstack/{ds}')
         )
