@@ -3,10 +3,16 @@ import ir_datasets
 from ir_datasets.util import Cache, DownloadConfig, GzipExtract, Lazy, Migrator
 from ir_datasets.datasets.base import Dataset, YamlDocumentation, FilteredQueries, FilteredScoredDocs, FilteredQrels
 from ir_datasets.formats import TrecDocs, TsvQueries, TrecQrels, TrecScoredDocs
-from ir_datasets.datasets.msmarco_passage import DUA, QRELS_DEFS, DL_HARD_QIDS_BYFOLD, DL_HARD_QIDS
+from ir_datasets.datasets.msmarco_passage import DUA, DL_HARD_QIDS_BYFOLD, DL_HARD_QIDS
+
+NAME = 'msmarco-document'
 
 _logger = ir_datasets.log.easy()
 
+QRELS_DEFS = {
+    1: 'Document contains a passage labeled as relevant in msmarco-passage'
+}
+
 TREC_DL_QRELS_DEFS = {
     3: "Perfectly relevant: Document is dedicated to the query, it is worthy of being a top result "
        "in a search engine.",
@@ -29,7 +35,7 @@ class MsMarcoDocument(NamedTuple):
 # Use the TREC-formatted docs so we get all the available formatting (namely, line breaks)
 class MsMarcoTrecDocs(TrecDocs):
     def __init__(self, docs_dlc):
-        super().__init__(docs_dlc, parser='text', lang='en', docstore_size_hint=14373971970, count_hint=3213835)
+        super().__init__(docs_dlc, parser='text', lang='en', docstore_size_hint=14373971970, count_hint=ir_datasets.util.count_hint(NAME))
 
     @ir_datasets.util.use_docstore
     def docs_iter(self):
@@ -52,9 +58,9 @@ class MsMarcoTrecDocs(TrecDocs):
 
 
 def _init():
-    base_path = ir_datasets.util.home_path()/'msmarco-document'
-    documentation = YamlDocumentation('docs/msmarco-document.yaml')
-    dlc = DownloadConfig.context('msmarco-document', base_path, dua=DUA)
+    base_path = ir_datasets.util.home_path()/NAME
+    documentation = YamlDocumentation(f'docs/{NAME}.yaml')
+    dlc = DownloadConfig.context(NAME, base_path, dua=DUA)
     subsets = {}
     collection = MsMarcoTrecDocs(GzipExtract(dlc['docs']))
 
@@ -163,9 +169,9 @@ def _init():
         documentation('trec-dl-hard/fold5')
     )
 
-    ir_datasets.registry.register('msmarco-document', Dataset(collection, documentation("_")))
+    ir_datasets.registry.register(NAME, Dataset(collection, documentation("_")))
     for s in sorted(subsets):
-        ir_datasets.registry.register(f'msmarco-document/{s}', Dataset(subsets[s], documentation(s)))
+        ir_datasets.registry.register(f'{NAME}/{s}', Dataset(subsets[s], documentation(s)))
 
     return collection, subsets
 
