@@ -33,7 +33,7 @@ def _init():
     dlc = DownloadConfig.context(NAME, base_path, dua=DUA)
     subsets = {}
 
-    collection = TrecDocs(dlc['docs'], path_globs=['**/FBIS/FB*', '**/FR94/??/FR*', '**/FT/*/FT*', '**/LATIMES/LA*'], namespace=NAME, lang='en', expected_file_count=2295, count_hint=528155)
+    collection = TrecDocs(dlc['docs'], path_globs=['**/FBIS/FB*', '**/FR94/??/FR*', '**/FT/*/FT*', '**/LATIMES/LA*'], namespace=NAME, lang='en', expected_file_count=2295, count_hint=ir_datasets.util.count_hint(NAME))
 
     queries = TrecQueries(GzipExtract(dlc['queries']), namespace=NAME, lang='en')
     qrels = TrecQrels(dlc['qrels'], QREL_DEFS)
