@@ -1,5 +1,6 @@
 import re
 import ir_datasets
+from .metadata import MetadataComponent
 
 
 __all__ = 'Registry'
@@ -17,8 +18,8 @@ class Registry:
             for pattern, initializer in self._patterns:
                 match = pattern.match(key)
                 if match:
-                    dataset = initializer(match.groups())
-                    self._registered[key] = dataset
+                    dataset = initializer(key, match.groups())
+                    self.register(key, dataset)
                     break
         return self._registered[key]
 
@@ -26,11 +27,14 @@ class Registry:
         return iter(self._registered.keys())
 
     def register(self, name, obj):
+        from ..datasets.base import Dataset
         if name in self._registered:
             if self._allow_overwrite:
                 _logger.warn(f"{name} already exists in this registry. Overwriting.")
             else:
                 raise RuntimeError(f"{name} already exists in this registry.")
+        if not hasattr(obj, 'metadata'):
+            obj = Dataset(MetadataComponent(name, obj), obj) # add metadata from default provider
         self._registered[name] = obj
 
     def register_pattern(self, pattern, initializer):
